class Category < ActiveRecord::Base

  has_many :spot_categories, :dependent => :destroy, :validate => false
  
  has_many :spots, :through => :spot_categories
  
  has_many :children,
    :dependent => :destroy,
    :class_name => "Category",
    :foreign_key => :parent_id
    
  
  # 親カテゴリが0のものを取得(トップ)
  scope :tops, where("parent_id = ?", 0)

  # children
  def sub_categories
    # organization_idは子カテゴリの検索をするときは未指定でも良いのでは？
    Category.where("organization_id = ? and parent_id = ?", self.organization_id, self.id).order("sort_number").all
#    Category.find(:all, :conditions => ["organization_id = ? and parent_id = ?", self.organization_id, self.id], :order => "sort_number")
  end
  
  def near_dist_catego_spot(lat, longi)
    categories = []
    if self.spot_categories.present?
      self.spot_categories.each do |spot_category|
        spot = spot_category.spot
          if spot.near_range?(lat, longi)
            dist = spot.distance(lat,longi)
            categories << [spot, dist]
          end
      end
    end
    return categories
  end

  def near_all_dist_catego_spots(lat, longi)
    categories = []
    if self.spot_categories.present?
      self.spot_categories.each do |spot_category|
        spot = spot_category.spot
          if spot.near_range?(lat, longi)
              dist = spot.distance(lat,longi)
              categories << [dist, spot, self]
          end
      end
    end
    return categories
  end

  def the_spots_near_all(lat, longi, spot_id=nil)
    categories = []
    if self.spot_categories.present?
      self.spot_categories.each do |spot_category|
        spot = spot_category.spot
        if spot.near_all_range?(lat, longi)
          if spot_id.nil? || (spot.id != spot_id)
            dist = spot.distance(lat,longi)
            categories << [dist, spot, self]
          end
        end
      end
    end
    return categories
  end
  
  def top?
    self.parent_id == 0
  end

end
