charset='utf-8'

google.load("maps","3",{other_params : 'sensor=true'});
google.load("jquery","1.4");
google.load("gdata","2");

var directionDisplay;
var directionsService;
var map;
var noResult = "<p style='padding:7px 0 0'>現在地周辺で登録されているおすすめスポットはありません。</p>";
var tryTime = 3;          // 現在地取得回数
var acceptableVal = 100;  // 採用する現在地情報の誤差許容値

// multipleFlag により、現在地の取得回数を１回のみ／複数回を切り分ける
function getClientLocation(stLatitude, stLongitude, multipleFlag) {
    if(multipleFlag){
      getLocationByWatchPosition(stLatitude, stLongitude);
    }else{
      getClientLocationOnce(stLatitude, stLongitude);
    }
}

// 現在地取得後、getLocationCallBack()を実行する
function getClientLocationOnce(stLatitude, stLongitude) {
    if(navigator.geolocation){
        navigator.geolocation.getCurrentPosition(
            function (val){
                getLocationCallBack(val.coords.latitude,val.coords.longitude);
                gpsInfoDisplay(val.coords.latitude,val.coords.longitude,val.coords.accuracy);
                LocationDisplay(val.coords.latitude,val.coords.longitude,val.coords.accuracy,val.timestamp);
            },
            function(){
                getLocationCallBack(stLatitude, stLongitude);
                LocationDisplay(stLatitude, stLongitude, null, null);
            },
            {maximumAge: 0, enableHighAccuracy: true}
        )
    }else{
        getLocationCallBack(stLatitude, stLongitude);
        LocationDisplay(stLatitude,stLongitude, null, null);
    }
}

// 現在地を複数回取得後、getLocationCallBack()を実行する
function getLocationByWatchPosition(stLatitude, stLongitude){
    var i = 1;
    var lastPos = null;
    if(navigator.geolocation){
        var watchId = navigator.geolocation.watchPosition(
            function (val){
                if(val.coords.accuracy <= acceptableVal || i >= tryTime){
                    getLocationCallBack(val.coords.latitude,val.coords.longitude);
                    gpsInfoDisplay(val.coords.latitude,val.coords.longitude,val.coords.accuracy);
                    LocationDisplay(val.coords.latitude,val.coords.longitude,val.coords.accuracy,val.timestamp);
                    navigator.geolocation.clearWatch(watchId);
                }else{
                    lastPos = val;
                    i++;
                }
            },
            function(){
                if(i >= tryTime){
                    navigator.geolocation.clearWatch(watchId);
                    if(lastLocation != null){
                        getLocationCallBack(lastPos.coords.latitude,lastPos.coords.longitude);
                        gpsInfoDisplay(lastPos.coords.latitude,lastPos.coords.longitude,lastPos.coords.accuracy);
                        LocationDisplay(lastPos.coords.latitude,lastPos.coords.longitude,lastPos.coords.accuracy,lastPos.timestamp);
                    }else{
                        getLocationCallBack(stLatitude, stLongitude);
                        LocationDisplay(stLatitude,stLongitude, null, null);
                    }
                }else{
                    i++;
                }
            },
            {maximumAge: 0,timeout:10000, enableHighAccuracy: true}
        )
    }else{
        getLocationCallBack(stLatitude, stLongitude);
        LocationDisplay(stLatitude,stLongitude, null, null);
    }
}

function gpsInfoDisplay(lat,longi,accuracy){
    var info = "(" + lat + ", " + longi + ", " + accuracy + ")";
    $("#location_infomation").text(info);
    $("#location_infomation").show();
}

function LocationDisplay(lat,longi,accuracy,time){
    var msgArea = $("#message_area");
    var mapUrl = "http://maps.google.co.jp/maps?q=" + lat + "," + longi;
    $(".link_button#go_map").attr("alt", mapUrl);
    if(accuracy != null){
        var timeObj = new Date(time);
        var accuracyNum = accuracy - 0;
        $("span#time").text("（" + datePlusZero(timeObj.getHours()) + "時" + datePlusZero(timeObj.getMinutes()) + "分取得）");
        if(acceptableVal < accuracyNum && accuracyNum <= 10000){
            var dispAcc = Math.floor((accuracy - 0) / 100) * 100;
            $("span#notice").append("※この現在地は" + dispAcc + "m以上の誤差が生じています<br />");
        }else if(10000 < accuracyNum){
            $("span#notice").append("※正確な現在地が取得できていません<br />");
        }
    }else{
        $("span#notice").append("※現在地が取得できないため、以下の場所を仮表示しています<br />");
        $("a#reload").hide();
    }
    // 逆ジオコーディング
    var geocoder = new google.maps.Geocoder();
    geocoder.geocode({'latLng': new google.maps.LatLng(lat, longi)}, function(res, status) {
        if (status == google.maps.GeocoderStatus.OK) {
            if (res) {$("span#address").text(res[1].formatted_address);}
        } else {
            $("span#address").text("取得に失敗しました");
        }
    });
    msgArea.slideDown("normal");
}

// スポットまでの直線距離を取得
function linearDistanceToSpot(spotLatitude,spotLongitude,meansRoute,tresDistance,lat,longi) {
    $.get("../get_linear_distance",
        {spot_lat: spotLatitude, spot_longi: spotLongitude, lat: lat, longi: longi},
        function(distance){
            $("#route_to").html('　ここから約' + distance　+ 'km（直線距離）');
            if($("#go_treasure").css("display") != undefined){
                var objGray = $("#no_treasure");
                var objRed = $("#go_treasure");
                if (distance <= ((tresDistance - 0) / 1000)){
                    objGray.css("display","none");
                    objRed.css("display","inline");
                }else{
                    objGray.css("display","inline");
                    objRed.css("display","none");
                }
            }
            makeMap(spotLatitude,spotLongitude,lat,longi,meansRoute,distance);
        }
    )
}

// イベント開催地までの直線距離を取得
function linearDistanceToEvent(eventLatitude,eventLongitude,meansRoute,lat,longi) {
    $.get("../spots/get_linear_distance",
        {spot_lat: eventLatitude, spot_longi: eventLongitude, lat: lat, longi: longi},
        function(distance){
            $("#route_to").html('　ここから約' + distance　+ 'km（直線距離）');
            makeMap(eventLatitude,eventLongitude,lat,longi,meansRoute,distance);
        }
    )
}

// 引数の緯度経度を中心とし、マーカーをつけた地図を出力する
function makeMap(destLat,destLongi,lat,longi,meansRoute,distance){
//    if (meansRoute == 'DRIVING'){
//        var zoom_num = 14;
//    }
//    if (meansRoute == 'WALKING'){
//        var zoom_num = 16;
//    }

    distance = distance - 0;
    var baseDistance = 0.13;  // 基準となる距離
    var zoomNum = 17;         // 採用する最も詳細なズームレベル
    var zoomNumStep = 15;     // ズームレベルの段階数
    // 2点間の距離を「基準となる距離 × 2のべき乗」と比較して地図のズームレベルを決定する
    for(var i=0;i < zoomNumStep;i++){
        if(distance <= (baseDistance * Math.pow(2, i))){
            break;
        }else{
            zoomNum -= 1;
        }
    }
    // 地図の横幅が450px以上なら、ズームレベルを1つ上げる
    if(($("#map").css("width").replace("px", "") - 0) >= 450){zoomNum++;}

    var directionsDisplay = new google.maps.DirectionsRenderer();
    var latLng = new google.maps.LatLng(destLat,destLongi);
    var centerPos = new google.maps.LatLng(((destLat - 0) + (lat - 0))/2,((destLongi - 0) + (longi - 0))/2);
    var myOptions = {
        zoom: zoomNum,
        center: centerPos,
        mapTypeId: google.maps.MapTypeId.ROADMAP,
	disableDefaultUI: false,
	mapTypeControl: false,
	navigationControl: true,
	scaleControl: true,
	streetViewControl: false
    }
    map = new google.maps.Map(document.getElementById('map'), myOptions);
    var destMarker = new google.maps.Marker({map: map, position: latLng});
    var currentMarker = new google.maps.Marker({
        map: map,
        position: new google.maps.LatLng(lat,longi),
        icon: "http://maps.google.co.jp/mapfiles/ms/icons/blue.png",
        shadow:new google.maps.MarkerImage(
            "http://www.google.com/mapfiles/shadow50.png",
            new google.maps.Size(37,34), // size
            new google.maps.Point(0,0),  // origin
            new google.maps.Point(12,35) // ancho
        )
    });
    directionsDisplay.setMap(map);
}

// よりみちスポット一覧を取得・表示
function yorimichiList(latitude,longitude){
    var division_id = document.getElementById("division_id").value;
    $.post("yorimichi_sort", {division_id: division_id, lat: latitude, longi: longitude}, function(result){
        $("#location").html(result);
        $("#loading_msg").fadeOut("slow");
    })
}

// 全スポット一覧
function callAllNears(latitude,longitude){
    $.post("all_nears", {lat : latitude, longi : longitude}, function(result){
        if(result == ""){result = noResult};
        $("#location").html(result);
        $("#loading_msg").fadeOut("normal");
    })
}

// カテゴリー別、サブカテゴリー別スポット一覧
function callCategoryNears(latitude,longitude,postUrl){
    var category_id = document.getElementById("ca_id").value;
    $.post(postUrl, {category_id : category_id, lat : latitude, longi : longitude}, function(result){
        if(result == ""){result = noResult};
        $("#location").html(result);
    })
}

//get two month later, and end day of the month
function twoMonthEnd(theday){
    var nowMonth = theday.getMonth();
    var tmlMonth = 0;
    var tmlYear = 0;
    if (nowMonth < 10){
        tmlMonth += theday.getMonth() + 3;
        tmlYear += theday.getFullYear();
    }else{
        tmlMonth += theday.getMonth() - 8;
        tmlYear += theday.getFullYear() + 1;
    }
    var endday = new Date(tmlYear, tmlMonth, 0);
    return endday;
}

// イベント情報取得
function getMyFeed(limit) {
    var today = new Date();
    var endday = twoMonthEnd(today);
    var service = null;
    var url = 'http://www.google.com/calendar/feeds/hikawanokanko%40gmail.com/public/full';
    service = new google.gdata.calendar.CalendarService("hikawanokanko@gmail.com");
    query = new google.gdata.calendar.CalendarEventQuery(url);
    query.setOrderBy(google.gdata.calendar.CalendarEventQuery.ORDERBY_START_TIME);
    // 最も未来のイベントから順番に取得する。
    query.setSortOrder(google.gdata.calendar.CalendarEventQuery.SORTORDER_ASCENDING);
    query.setSingleEvents(true);
    if(limit != null){
        // 最も未来のイベントから指定された件数だけ取得する
        query.setMaxResults(limit);
    }
    var start_date = dateChange(today);
    var end_date = dateChange(endday);
    var start = new google.gdata.DateTime.fromIso8601(start_date);
    var end = new google.gdata.DateTime.fromIso8601(end_date);
    query.setMinimumStartTime(start);
    query.setMaximumStartTime(end);
    service.getEventsFeed(query, callback, handleError);
}

function dateChange(date){
    var date_str = '';
    var day = date.getDate();
    day = datePlusZero(day);
    var month = date.getMonth() + 1;
    month = datePlusZero(month);
    date_str += date.getFullYear() + '/' + month + '/' + day;
    return date_str;
}

function handleError(error){
    alert(error);
}

function callback(result){
    var events = new Array();
    var entries = result.feed.entry.reverse();
    for (var i = 0; i < entries.length; i++) {
	var entry = entries[i];
        var address = "";
        if (entry.gd$where[0].valueString){
          address += entry.gd$where[0].valueString;
        }
	var title = entry.getTitle().getText();
	var times = entry.getTimes();
	if(times.length > 0){
	    var start_time = google.gdata.DateTime.fromIso8601(times[0].startTime);
	    var end_time = google.gdata.DateTime.fromIso8601(times[0].endTime);
            var start_date = dateStr(start_time);
            var end_date = dateStr(end_time);
            var startEndTime = "";
            // 終日でなければ、イベント開始・終了時刻を取得
            if(start_time.getDate().getHours() != 0 || start_time.getDate().getMinutes() != 0 ||
                end_time.getDate().getHours() != 0 || end_time.getDate().getMinutes() != 0){
                var startHour = start_time.getDate().getHours();
                var startMin = datePlusZero(start_time.getDate().getMinutes());
                var endHour = end_time.getDate().getHours();
                var endMin = datePlusZero(end_time.getDate().getMinutes());
                startEndTime = startHour + ":" + startMin + " 〜 " + endHour + ":" + endMin;
            }
            // イベント期間が１日だけの場合
            if(start_time.getDate().getMonth() == end_time.getDate().getMonth() &&
                start_time.getDate().getDate() == end_time.getDate().getDate() ){
                  end_date = "";
            }

            var placeHtml = "";
            if (entry.gd$where[0].valueString){
                placeHtml = address;
            }
            var event = {
                title : title,
                start_date : start_date,
                end_date : end_date,
                time : startEndTime,
                place : placeHtml,
                detail : entry.getContent().getText()
            };
            events.push(event);
        }
    }
    if(events.length == 0){
        $("#location").html("<p style='padding:5px;color:#333'>現在イベント情報はありません。</p>");
    }else{
        var url = $("input[name='url']").val();
        $.post(url, {events : events}, function(result){
            $("#location").html(result);
        })
    }
}

function dateStr(date){
    var date_str = '';
    var day = date.getDate().getDate();
    day = datePlusZero(day);
    var month = date.getDate().getMonth() + 1;
    month = datePlusZero(month);
    date_str += date.getDate().getFullYear() + '/' + month + '/' + day;
    return date_str;
}

function datePlusZero(val){
    if (val < 10){
        val = '0' + val;
    }else{
        val += '';
    }
    return val;
}

// 現在位置を取得し、googleMapへ移動する
// 未使用
function jumpRouteMap(spot_latitude,spot_longitude,means_route,st_latitude,st_longitude){
    if (means_route == 'DRIVING'){
        var difflag = '&dirflg=d';
    }
    if (means_route == 'WALKING'){
        var difflag = '&dirflg=w';
    }
    var baseUrl = 'http://maps.google.com/maps?saddr=';
    var optionUrl = '&daddr=' + spot_latitude + ',' + spot_longitude + difflag;
    if(navigator.geolocation){
        navigator.geolocation.getCurrentPosition(
            function (val){
                location.href = baseUrl + val.coords.latitude + ',' + val.coords.longitude + optionUrl;
//                startendRoute(val.coords.latitude,val.coords.longitude);
            },
            function (){
                location.href = baseUrl + st_latitude + ',' + st_longitude + optionUrl;
//                alert('位置情報取得に失敗しました');
            },
            {maximumAge: 0, enableHighAccuracy: true}
        );
    }else{
        location.href = baseUrl + st_latitude + ',' + st_longitude + optionUrl;
//        alert("ご使用の端末ではルート案内を行うことができません");
    }
}

// googleMapのurlを生成、aタグに挿入する
function setRouteMapUrl(spotLatitude,spotLongitude,meansRoute,latitude,longitude){
    if (meansRoute == 'DRIVING'){
        var difflag = '&dirflg=d';
    }
    if (meansRoute == 'WALKING'){
        var difflag = '&dirflg=w';
    }
    var baseUrl = 'http://maps.google.com/maps?saddr=';
    var optionUrl = latitude + ',' + longitude + '&daddr=' + spotLatitude + ',' + spotLongitude + difflag;

    $("#route_map_button").attr("href", baseUrl + optionUrl);
}


// イベント開催地の緯度経度を取得後、イベント情報をサブミットする
function getLocationBeforeSubmit(formName){
    var address = $("form[name='"+formName+"'] #place").val()
    geocoder = new google.maps.Geocoder();
    if (geocoder) {
        geocoder.geocode( {'address': address}, function(results, status) {
            if (status == google.maps.GeocoderStatus.OK) {
                $("form[name='"+formName+"'] #latitude").val(results[0].geometry.location.lat());
                $("form[name='"+formName+"'] #longitude").val(results[0].geometry.location.lng());
                $("form[name='"+formName+"']").submit();
            }else{
                $("form[name='"+formName+"']").submit();
            }
        })
    }
}


// スポットまでのルート距離を取得、表示する
// （未使用）
function theSpotRoute(spot_latitude,spot_longitude,means_route,tres_distance,st_latitude,st_longitude,lat,longi) {
    var directionsService = new google.maps.DirectionsService();
    var start = "";
    if(lat == 0 || longi == 0){
        start += st_latitude + ',' + st_longitude;
    }else{
        start += lat + ',' + longi;
    }
    var end = spot_latitude + ',' + spot_longitude;
    var limit_distance = (tres_distance - 0) / 1000;
    var request = {
        origin:start,
        destination:end,
        travelMode:means_route
    };
      directionsService.route(request, function(result, status) {
        if (status == google.maps.DirectionsStatus.OK) {
          var distance = Math.round((result.routes[0].legs[0].distance.value)/100) / 10.0;
          $("#route_to").html('　ここから約' + distance　+ 'km');
          if($("#go_treasure").css("display") != undefined){
              var objGray = document.getElementById("no_treasure");
              var objRed = document.getElementById("go_treasure");
              if (distance <= limit_distance){
                  objGray.style.display = "none";
                  objRed.style.display = "inline";
              }else{
                  objGray.style.display = "inline";
                  objRed.style.display = "none";
              }
          }
          makeMap(spot_latitude,spot_longitude,means_route);
        }
      });
}

// イベント開催地までのルート距離を取得、表示する
// （未使用）
function theEventRoute(title,start_date,end_date,place,detail,means_route,st_latitude,st_longitude,lat,longi) {
    var directionsService = new google.maps.DirectionsService();
    var start = "";
    if(lat == 0 || longi == 0){
        start += st_latitude + ',' + st_longitude;
    }else{
        start += lat + ',' + longi;
    }
    var end = place;
    var request = {
        origin:start,
        destination:end,
        travelMode:means_route
    };
      directionsService.route(request, function(result, status) {
        if (status == google.maps.DirectionsStatus.OK) {
          var distance = Math.round((result.routes[0].legs[0].distance.value)/100) / 10.0;
          getGeoCode(title,start_date,end_date,place,detail,means_route,st_latitude,st_longitude,distance);
        }
     });
}